@@ -17,10 +17,10 @@ import (
 	"io"
 	"time"
 
-	"gopkg.in/alecthomas/kingpin.v2"
+	kingpin "gopkg.in/alecthomas/kingpin.v2"
 
-	"github.com/prometheus/alertmanager/client"
-	"github.com/prometheus/alertmanager/types"
+	"github.com/go-openapi/strfmt"
+	"github.com/prometheus/alertmanager/api/v2/models"
 )
 
 const DefaultDateFormat = "2006-01-02 15:04:05 MST"
@@ -36,14 +36,15 @@ func InitFormatFlags(app *kingpin.Application) {
 // Formatter needs to be implemented for each new output formatter.
 type Formatter interface {
 	SetOutput(io.Writer)
-	FormatSilences([]types.Silence) error
-	FormatAlerts([]*client.ExtendedAlert) error
-	FormatConfig(*client.ServerStatus) error
+	FormatSilences([]models.GettableSilence) error
+	FormatAlerts([]*models.GettableAlert) error
+	FormatConfig(*models.AlertmanagerStatus) error
+	FormatClusterStatus(status *models.ClusterStatus) error
 }
 
 // Formatters is a map of cli argument names to formatter interface object.
 var Formatters = map[string]Formatter{}
 
-func FormatDate(input time.Time) string {
-	return input.Format(*dateFormat)
+func FormatDate(input strfmt.DateTime) string {
+	return time.Time(input).Format(*dateFormat)
 }
